#!/bin/bash
# ircDDB Install Script for Standard DStar Gateways
#
# Hans-J. Barthen, DL5DI
# Franz-Josef-Str. 20
# D 56642 Kruft
#
# Tel.: +49 2652 938377
# Skype: hjbarthen / dl5di
# Email: dl5di@darc.de
#
# 2010/06/02 Version 0.91
# 2010/06/14 Version 0.92 change autostart from monstart.sh to /etc/init.d/dstar_gw (otherwise restart from gw_schedule won't work, PG is blocked)
# 2010/06/15 Version 0.93 bugfix in installation of DSM database driver (double index)
# 2010/07/03 Version 0.94 change to new server, paths and classpath modified
# 2010/07/05 Version 0.95 bugfix (x-bits runs.sh, name ircDBD-vs,  download path)
#

# Installation information
IRCDDBUSER="ircddb"
IRCDDBHOME="/opt/ircDDB"
IRCDDBINST="/opt/ircDDB/install"
#
# Download-URL
CURL="/usr/bin/curl -4 --connect-timeout 30 --max-time 240"
DOWNLOAD_URL="http://download.ircddb.net/ircddb-icom-dsm/install"
IRCDDBINIT="install2-ircddb.tgz"
#
# Database settings
IRCSRV="group1-irc.ircddb.net"
IRCPORT="9007"
DATABASE="//group1-postgres.ircddb.net:5432/ircddb"
#
# IRC settings
IRCCHAN="#dstar"
UPDURL="http://group1-update.ircddb.net/ircDDB"
#
# Other setups
JAVPGCONN="postgresql-8.4-701.jdbc3.jar"
DSICONF=/opt/products/dstar/dstar_gw/dsipsvd/dsipsvd.conf
#
#####################################################################

if [ $EUID != 0 ] ; then
    echo "Sorry, the install script must be run as root!"
    echo "Aborting."
    echo " "
    exit
fi

IRCPWD=$1
AUTO=0

if [ $IRCPWD ]; then
    AUTO=1
fi

echo " "
echo "ircDDB Install Script for Standard DStar Gateways"
echo " "
echo "Copyright (c) 2010"
echo "Michael Dirska - DL1BFF, Jann Traschewski - DG8NGN, Hans-J. Barthen - DL5DI"
echo " "
echo "This program is free software; you can redistribute it and/or modify"
echo "it under the terms of the GNU General Public License as published by"
echo "the Free Software Foundation; either version 2 of the License, or"
echo "(at your option) any later version."
echo "  "
echo "This program is distributed in the hope that it will be useful,"
echo "but WITHOUT ANY WARRANTY; without even the implied warranty of"
echo "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the"
echo "GNU General Public License for more details."
echo "	"
echo "You should have received a copy of the GNU General Public License"
echo "along with this program; if not, write to the Free Software"
echo "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"
echo " "
echo "DG8NGN, DL1BFF, DL5DI  28.05.2010"
echo " "

if [ $AUTO -eq 0 ]; then
# ask for password, it was not supplied in the command line

    while [ ! $IRCPWD ]; do
	echo " "
	echo "Please enter the password provided by the ircDDB-team to enter the system:"
	echo " "
	read IRCPWD
    done
fi

echo " "
echo "OK, Password will be set to '$IRCPWD'"

# check if GW_CALL could be found

echo " "
echo "Check requirements:"
echo " "
#
GW_CALL=`egrep -e '^ZR_CALLSIGN' $DSICONF | sed -e 's/ //g' | awk -F"=" '{print $2}' | tr '[A-Z]' '[a-z]' | sed -e 's/\r//'`

Dat=`date +%Y%m%d`

if [ ! $GW_CALL ]; then 
    echo "- Missing GW-Call!"
    echo "  This installscript is for a standard Icom Gateway-Installation only!"
    echo -n "You may try it on other systems by entering the call manually (lower case!):"
    read GW_CALL
    if [ ! $GW_CALL ]; then 
	exit
    fi
fi

echo "+ Gateway Callsign set to '$GW_CALL'"

# check if DSM is installed
DSMHOME=`find /opt -name dstarmon`

if [ ! $DSMHOME ]; then 
    echo "- Missing DStarMonitor!"
    exit
fi

echo "+ DStarMonitor found at $DSMHOME !"

# check for Java-directory
JAVABIN=`egrep -e '^javadir' $DSMHOME/monstart.sh | sed -e 's/ //g' | awk -F"=" '{print $2}' |sed -e 's/\r//'`

if [ ! $JAVABIN ]; then 
    echo "- Missing Java!"
    echo "  This installscript is for a standard Icom Gateway-Installation!"
    exit
fi

echo "+ Java found at $JAVABIN !"

# check where Postgres is installed
PGSQL=`find /opt -name psql`

if [ ! $PGSQL ]; then 
    echo "  The installscript is for a standard Icom Gateway-Installation!"
    echo "  This system seems to be different. We will extend our search, it may take a while ...."
    PGSQL=`find / -name psql`
fi

if [ ! $PGSQL ]; then 
    echo "- Missing Postgres, sorry, no chance!"
    exit
fi

echo "+ Postgres found at $PGSQL !"
echo " "
echo "All requirements seem to be OK!"
echo " "

# check if ircDDB is already installed and should be updated

if [ -d $IRCDDBHOME ]; then 

    echo "- ircDDB is obviously already installed on your system!"
    echo " "
    echo "1. Refresh/Update installation"
    echo "2. Delete installation and quit"
    echo "3. Quit"
    echo " "
    echo -n "Please choose selection followed by ENTER: "
    choice=4
    
    while [ $choice -eq 4 ]; do

	if [ $AUTO -eq 0 ]; then
	    read choice
	else
	    choice=1
	fi

	if [ $choice -eq 1 ]; then
	    echo "+ Refreshing existing installation"
	    rm -f $IRCDDBHOME/*;
	else
	    if [ $choice -eq 2 ]; then
		echo "+ Removing existing installation and exiting"
		rm -Rf $IRCDDBHOME;
		exit
	    else
		if [ $choice -eq 3 ]; then
		    echo "+ Exiting without any changes"
    		    exit
		else
		    echo "- ircDDB is already installed on your system!"
		    echo " "
		    echo "1. Refresh/Update installation"
		    echo "2. Delete installation and quit"
		    echo "3. Quit"
		    echo " "
		    echo -n "Please choose selection followed by ENTER: "
		    choice=4
		fi
	    fi
	fi
    done
fi

echo " "
#
echo "Starting installation of ircDDB Init-version on $GW_CALL"
#
echo "+ Create new user '$IRCDDBUSER'"
useradd -c "ircDDB-User" -d $IRCDDBHOME $IRCDDBUSER
#
if [ ! -d $IRCDDBHOME ]; then
    echo "+ Create home directory '$IRCDDBHOME'"
    mkdir $IRCDDBHOME
fi
#
if [ ! -d $IRCDDBINST ]; then
    echo "+ Create install directory '$IRCDDBINST'"
    mkdir $IRCDDBINST
fi
#
cd $IRCDDBINST
DL=1
if [ -f $IRCDDBINIT ]; then 
    choice=3
    echo " "
    echo "- The initial installation file already exists at $IRCDDBINST/$IRCDDBINIT"
    echo " "
    echo "1. Use the existing file"
    echo "2. Start new Download"
    echo " "
    echo -n "Please choose selection followed by ENTER: "

    while [ $choice -eq 3 ]; do

	if [ $AUTO -eq 0 ]; then
	    read choice
	else
	    choice=2
	fi

	if [ $choice -eq 1 ]; then
	    echo "+ Using the existing file "
	    DL=0
	else
	    if [ $choice -ne 2 ]; then
		echo " "
		echo "- The initial installation file already exists at $IRCDDBINST/$IRCDDBINIT"
	        echo " "
	        echo "1. Use the existing file"
	        echo "2. Start new Download"
	        echo " "
		echo -n "Please choose selection followed by ENTER: "
		choice=3
	    fi	
	fi
    done
fi

if [ $DL -eq 1 ]; then
    echo " "
    echo "+ Download initial installation file '$IRCDDBINIT' to '$IRCDDBINST'"
    echo "Please wait!!"
    echo " "
# delete really all before we start
    rm -f $IRCDDBINST/*
    $CURL -O $DOWNLOAD_URL/$IRCDDBINIT 2>/dev/null
fi

#
echo " "
echo "unpack tar-file"
echo " "
cd /
tar -zxvf $IRCDDBINST/$IRCDDBINIT
echo " "
#
echo "+ Stop DStarMonitor"
/etc/init.d/dsm stop
echo "+ Stop all processes run by user '$IRCDDBUSER'"
killall -q -u $IRCDDBUSER
#
# add ircDDB database server to DSM properties
echo "+ Saving dstarmonitor.properties"
cp $DSMHOME/dstarmonitor.properties $DSMHOME/dstarmonitor.properties.ircddb-bak.$Dat
#
echo "+ Saving monstart.sh"
cp $DSMHOME/monstart.sh $DSMHOME/monstart.sh.ircddb-bak.$Dat
echo "+ Adding Java-Postgres-Connector to class"
cat $DSMHOME/monstart.sh | grep $JAVPGCONN > /dev/null || sed -i "s/ Main/:$JAVPGCONN Main/g" $DSMHOME/monstart.sh
#
echo "+ Installing Java-Postgres-Connector"
cp $IRCDDBINST/$JAVPGCONN $DSMHOME
#
echo "+ Adding IRC-Database to dstarmonitor.properties."

# is our driver already installed at index 0 ?

LHPARM=`egrep -e "^LHParameters=user;$GW_CALL;password;$IRCPWD" $DSMHOME/dstarmonitor.properties | sed -e 's/\r//'`
if [ $LHPARM ]; then
    echo "+ Java-Postgres-Connector already installed at Index 0."
    LHURL=`egrep -e "^LHURI=jdbc:postgresql:$DATABASE" $DSMHOME/dstarmonitor.properties | sed -e 's/\r//'`
    if [ $LHURL ]; then
	echo "+ DSM-Settings are OK! We can skip this step."
    else
	echo "+ Set new server parameters in DSM-properties at Index 0."
        cat $DSMHOME/dstarmonitor.properties.ircddb-bak.$Dat | sed "/^LHURI=/cLHURI=jdbc:postgresql:$DATABASE" > $DSMHOME/dstarmonitor.properties
    fi
#
else

# our driver is not at index 0.
# Is any other driver installed at index 0 ?

    LHD=`egrep -e "^LHDriver=" $DSMHOME/dstarmonitor.properties | sed -e 's/ //g' | awk -F"=" '{print $2}' | sed -e 's/\r//'`
    if [ $LHD ]; then

# index 0 is already in use.
# Is our driver installed at index 1 ?

        LHPARM1=`egrep -e "LHParameters1=user;$GW_CALL;password;$IRCPWD" $DSMHOME/dstarmonitor.properties | sed -e 's/\r//'`
        if [ $LHPARM1 ]; then
	    echo "+ Java-Postgres-Connector already installed at Index 1."
	    LHURL1=`egrep -e "^LHURI1=jdbc:postgresql:$DATABASE" $DSMHOME/dstarmonitor.properties | sed -e 's/\r//'`
	    if [ $LHURL1 ]; then
    	        echo "+ DSM-Settings are OK! We can skip this step."
	    else
    	        echo "+ Set new server parameters in DSM-properties at Index 1."
    	        cat $DSMHOME/dstarmonitor.properties.ircddb-bak.$Dat | sed "/^LHURI1=/cLHURI1=jdbc:postgresql:$DATABASE" > $DSMHOME/dstarmonitor.properties
	    fi
	else

# Our driver is not at index 1
# Is any other additional LHDrivers is installed at index 1

	    LHD1=`egrep -e "^LHDriver1=" $DSMHOME/dstarmonitor.properties | sed -e 's/ //g' | awk -F"=" '{print $2}' | sed -e 's/\r//'`
	    if [ $LHD1 ]; then
    	        echo "- LHDriver1 already in use."
    	        echo "- Sorry, this is no default configuration, please try manual setup of the LHDriver!"
    	        echo "- Skipping DSM driver setup"
	    else
	        echo "+ Installing Java-Postgres-Connector at Driver Index 1"
	        cat $DSMHOME/dstarmonitor.properties.ircddb-bak.$Dat | sed "/^LHParameters=/a#\nLHDriver1=org.postgresql.Driver\nLHURI1=jdbc:postgresql:$DATABASE\nLHParameters1=user;$GW_CALL;password;$IRCPWD\n#" > $DSMHOME/dstarmonitor.properties
	    fi
	fi
    else
# there is no driver installed at index 0 yet
        echo "+ Installing Java-Postgres-Connector at Driver Index 0"
	cat $DSMHOME/dstarmonitor.properties.ircddb-bak.$Dat | sed "/^CtrlrIP=/a#\nLHDriver=org.postgresql.Driver\nLHURI=jdbc:postgresql:$DATABASE\nLHParameters=user;$GW_CALL;password;$IRCPWD\n#" > $DSMHOME/dstarmonitor.properties
    fi
fi


echo " "
echo "Configure ircDDB.properties"
echo " "
sed -i "s|<USERNAME>|$GW_CALL|g" $IRCDDBHOME/ircDDB.properties
sed -i "s|<PASSWORD>|$IRCPWD|g" $IRCDDBHOME/ircDDB.properties
sed -i "s|<IRCSRV>|$IRCSRV|g" $IRCDDBHOME/ircDDB.properties
sed -i "s|<IRCPORT>|$IRCPORT|g" $IRCDDBHOME/ircDDB.properties
sed -i "s|<IRCCHAN>|$IRCCHAN|g" $IRCDDBHOME/ircDDB.properties
echo " "
echo "+ Configure ircDDB run-script"
sed -i "s|<DSMHOME>|$DSMHOME|g" $IRCDDBHOME/run.sh
sed -i "s|<JAVABIN>|$JAVABIN|g" $IRCDDBHOME/run.sh
sed -i "s|<IRCSRV>|$IRCSRV|g" $IRCDDBHOME/run.sh
sed -i "s|<IRCPORT>|$IRCPORT|g" $IRCDDBHOME/run.sh
sed -i "s|<IRCCHAN>|$IRCCHAN|g" $IRCDDBHOME/run.sh
sed -i "s|<UPDURL>|$UPDURL|g" $IRCDDBHOME/run.sh
sed -i "s|<JAVPGCONN>|$JAVPGCONN|g" $IRCDDBHOME/run.sh
echo " "
echo "+ Configure ircDDB start-script"
sed -i "s|<IRCDDBHOME>|$IRCDDBHOME|g" $IRCDDBHOME/start.sh
#
echo " "
echo "+ Configure ircDDB.policy"
sed -i "s|<DSMHOME>|$DSMHOME|g" $IRCDDBHOME/ircDDB.policy
sed -i "s|<JAVPGCONN>|$JAVPGCONN|g" $IRCDDBHOME/ircDDB.policy
#
chown -R $IRCDDBUSER $IRCDDBHOME
echo " "
echo "+ Install database trigger"
echo " "
su postgres -c "$PGSQL -U dstar dstar_global < $IRCDDBINST/trigger.txt"
echo " "
echo "+ Install modified Startscript /etc/init.d/dstar_gw"
echo " "
sed -i "s|<IRCDDBHOME>|$IRCDDBHOME|g" $IRCDDBINST/dstar_gw
cp -f /etc/init.d/dstar_gw /etc/init.d/dstar_gw.ircddb-bak.$Dat
cp -f -p $IRCDDBINST/dstar_gw /etc/init.d
echo " "
echo "Installation complete!"
echo " "
echo "Ready to go ...."
echo " "


echo "+ Restart dstar_gw including ircDDB-client"
/etc/init.d/dstar_gw restart
echo "+ Restart dsm with new Properties"
/etc/init.d/dsm start

echo " "
echo " * * * Welcome to the DStar-ircDDB network ! * * *"
exit
